@@ -4,8 +4,9 @@ package # hide from PAUSE
 use strict;
 use warnings;
 use DBIx::ContextualFetch;
+use Sub::Name ();
 
-use base qw/DBIx::Class/;
+use base qw(Class::Data::Inheritable);
 
 __PACKAGE__->mk_classdata('sql_transformer_class' =>
                           'DBIx::Class::CDBICompat::SQLTransformer');
@@ -54,11 +55,13 @@ __PACKAGE__->mk_classdata('_transform_sql_handlers' =>
         $self->throw_exception( "No relationship to JOIN from ${from_class} to ${to_class}" )
           unless $rel_obj;
         my $join = $from_class->storage->sql_maker->_join_condition(
-          $from_class->result_source_instance->resolve_condition(
-            $rel_obj->{cond}, $to, $from) );
+          scalar $from_class->result_source_instance->_resolve_condition(
+            $rel_obj->{cond}, $to, $from
+          )
+        );
         return $join;
       }
-        
+
   } );
 
 sub db_Main {
@@ -79,18 +82,23 @@ sub __driver {
 sub set_sql {
   my ($class, $name, $sql) = @_;
   no strict 'refs';
-  *{"${class}::sql_${name}"} =
+  my $sql_name = "sql_${name}";
+  my $full_sql_name = join '::', $class, $sql_name;
+  *$full_sql_name = Sub::Name::subname $full_sql_name,
     sub {
       my $sql = $sql;
       my $class = shift;
-      return $class->storage->sth($class->transform_sql($sql, @_));
+      return $class->storage->dbh_do(
+        _prepare_sth => $class->transform_sql($sql, @_)
+      );
     };
   if ($sql =~ /select/i) {
-    my $meth = "sql_${name}";
-    *{"${class}::search_${name}"} =
+    my $search_name = "search_${name}";
+    my $full_search_name = join '::', $class, $search_name;
+    *$full_search_name = Sub::Name::subname $full_search_name,
       sub {
         my ($class, @args) = @_;
-        my $sth = $class->$meth;
+        my $sth = $class->$sql_name;
         return $class->sth_to_objects($sth, \@args);
       };
   }
@@ -111,7 +119,7 @@ sub sth_to_objects {
 
 sub transform_sql {
   my ($class, $sql, @args) = @_;
-  
+
   my $tclass = $class->sql_transformer_class;
   $class->ensure_class_loaded($tclass);
   my $t = $tclass->new($class, $sql, @args);
