@@ -10,7 +10,7 @@ sub page {
   my $class = shift;
 
   my $rs = $class->search(@_);
-  unless ($rs->{page}) {
+  unless ($rs->{attrs}{page}) {
     $rs = $rs->page(1);
   }
   return ( $rs->pager, $rs );
