@@ -3,6 +3,11 @@ package DBIx::Class::Exception;
 use strict;
 use warnings;
 
+# load Carp early to prevent tickling of the ::Internal stash being
+# interpreted as "Carp is already loaded" by some braindead loader
+use Carp ();
+$Carp::Internal{ (__PACKAGE__) }++;
+
 use DBIx::Class::Carp ();
 
 use overload
