@@ -9,7 +9,9 @@ no_index directory => $_ for (qw|
 |);
 no_index package => $_ for (qw/
   DBIx::Class::Storage::DBIHacks
+  DBIx::Class::Storage::BlockRunner
   DBIx::Class::Carp
+  DBIx::Class::_Util
   DBIx::Class::ResultSet::Pager
 /);
 
